# noinspection PyPep8Naming,PyMethodMayBeStatic
class MyJsonRPC:
    def testAction1(self):
        return {
            "action1": True
        }

    def testAction2(self):
        return {
            "action2": True
        }
